@@ -18,10 +18,13 @@ class StateData:
         self._entries[key] = value
 
     def __getitem__(self, key):
-        entry = self._entries[key]
-        if callable(entry):
-            return entry()
-        return entry
+        try:
+            entry = self._entries[key]
+            if callable(entry):
+                return entry()
+            return entry
+        except Exception:
+            return None
 
 
 class Format:
@@ -98,6 +101,11 @@ class AttrColumn(Column):
         self.attr = attr
 
     def content(self, state, _) -> str:
+        value = self.attr(state)
+
+        if value is None:
+            return "{0:>{1}}".format("n/a", self.width)
+
         return self.format(self.attr(state))
 
 
@@ -191,7 +199,11 @@ def iter_cols(problem):
             AttrColumn("Bound inf", 16, "{:16.8e}", IterateAttr("bound_violation"))
         )
 
-    cols.append(AttrColumn("Cons inf", 16, "{:16.8e}", IterateAttr("cons_violation")))
+    if problem.num_cons > 0:
+        cols.append(
+            AttrColumn("Cons inf", 16, "{:16.8e}", IterateAttr("cons_violation"))
+        )
+
     cols.append(AttrColumn("Dual inf", 16, "{:16.8e}", IterateAttr("stat_res")))
 
     return cols
