@@ -5,7 +5,7 @@ import logging
 import os
 from abc import ABC, abstractmethod
 from concurrent.futures import FIRST_COMPLETED, ProcessPoolExecutor, wait
-from multiprocessing import cpu_count
+from multiprocessing import Process, Queue, cpu_count
 
 import numpy as np
 
@@ -21,11 +21,10 @@ formatter = logging.Formatter("%(asctime)s:%(name)s:%(levelname)s:%(message)s")
 def solve_instance(instance, params, log_filename, verbose):
     logger.handlers.clear()
 
-    np.seterr(divide="raise", over="raise", invalid="raise")
-
-    handler = logging.FileHandler(log_filename)
-    handler.setFormatter(formatter)
-    logger.addHandler(handler)
+    if log_filename is not None:
+        handler = logging.FileHandler(log_filename)
+        handler.setFormatter(formatter)
+        logger.addHandler(handler)
 
     if verbose:
         handler = logging.StreamHandler()
@@ -39,34 +38,42 @@ def solve_instance(instance, params, log_filename, verbose):
     warn_logger.addHandler(handler)
     warn_logger.setLevel(logging.WARN)
 
-    return instance.solve(params)
-
-
-def try_solve_instance(instance, params, log_filename, verbose):
     try:
-        # No time limit
-        if params.time_limit == np.inf:
-            return (instance, solve_instance(instance, params, log_filename, verbose))
-
-        with ProcessPoolExecutor(1) as pool:
-            future = pool.submit(
-                solve_instance, instance, params, log_filename, verbose
-            )
-            done, _ = wait([future], timeout=(params.time_limit + 10))
-
-            if len(done) == 0:
-                logger.error("Reached timeout, aborting")
-                return (instance, "timeout")
-
-            result = next(iter(done)).result()
-            return (instance, result)
-
+        np.seterr(divide="raise", over="raise", invalid="raise")
+        result = instance.solve(params)
+        return (instance, result)
     except Exception as exc:
-        logger.error("Error solving %s", instance.name)
-        logger.exception(exc, exc_info=(type(exc), exc, exc.__traceback__))
+        logger.error("Error solving %s", instance.name, exc_info=exc)
         return (instance, "error")
 
 
+def solve_instance_queue(queue, instance, params, log_filename, verbose):
+    instance_result = solve_instance(instance, params, log_filename, verbose)
+    queue.put(instance_result)
+
+
+def solve_instance_time_limit(instance, params, log_filename, verbose):
+    if params.time_limit == np.inf:
+        return solve_instance(instance, params, log_filename, verbose)
+
+    queue = Queue()
+    process = Process(
+        target=solve_instance_queue,
+        args=(queue, instance, params, log_filename, verbose),
+    )
+
+    process.start()
+    process.join(timeout=(params.time_limit + 10.0))
+
+    if process.is_alive():
+        logger.error("Reached timeout, terminating process for %s", instance.name)
+        process.terminate()
+        logger.error("Terminated process for %s", instance.name)
+        return (instance, "timeout")
+    else:
+        return queue.get()
+
+
 class Runner(ABC):
     def __init__(self, name):
         self.name = name
@@ -87,13 +94,15 @@ class Runner(ABC):
         return params
 
     def log_filename(self, args, instance):
+        if args.no_output:
+            return None
         return self.output_filename(args, f"{instance.name}.log")
 
     def solve_instances_sequential(self, instances, args, params):
         verbose = args.verbose
         for instance in instances:
             log_filename = self.log_filename(args, instance)
-            yield try_solve_instance(instance, params, log_filename, verbose)
+            yield solve_instance_time_limit(instance, params, log_filename, verbose)
 
     def solve_instances_parallel(self, instances, args, params):
         verbose = args.verbose
@@ -114,16 +123,35 @@ class Runner(ABC):
         solve_args = zip(instances, all_params, all_log_filenames, all_verbose)
 
         with ProcessPoolExecutor(num_procs, max_tasks_per_child=1) as pool:
-            futures = [
-                pool.submit(try_solve_instance, *solve_arg) for solve_arg in solve_args
-            ]
 
-            while len(futures) != 0:
+            futures = []
+
+            for solve_arg in solve_args:
+                future = pool.submit(solve_instance_time_limit, *solve_arg)
+                future.instance = solve_arg[0]
+                futures.append(future)
+
+            run_logger.info("Started %d instances", len(futures))
+
+            while True:
                 done, futures = wait(futures, return_when=FIRST_COMPLETED)
 
                 for item in done:
                     yield item.result()
 
+                if len(futures) == 0:
+                    run_logger.info("Solved all instances, terminating")
+                    break
+
+                remaining_future = next(iter(futures))
+
+                run_logger.info(
+                    "Finished %d instances, waiting for %d (including %s)",
+                    len(done),
+                    len(futures),
+                    remaining_future.instance.name,
+                )
+
     def solve_instances(self, instances, args):
         # yields sequence of tuples of (instance, result) for each instance
         run_logger.info("Solving %d instances", len(instances))
@@ -184,6 +212,7 @@ class Runner(ABC):
         group = parser.add_argument_group(title="runner")
 
         parser.add_argument("--output", type=str)
+        parser.add_argument("--no_output", action="store_true")
         parser.add_argument("--max_size", type=int)
         parser.add_argument("--name", type=str)
         parser.add_argument("--unconstrained", action="store_true")
@@ -203,11 +232,14 @@ class Runner(ABC):
 
         args = self.parser().parse_args()
 
-        if args.output is None:
-            now = datetime.datetime.now().isoformat("T", "seconds")
-            args.output = f"output_{self.name}_{now}"
+        with_output = not (args.no_output)
 
-        os.makedirs(args.output, exist_ok=True)
+        if with_output:
+            if args.output is None:
+                now = datetime.datetime.now().isoformat("T", "seconds")
+                args.output = f"output_{self.name}_{now}"
+
+            os.makedirs(args.output, exist_ok=True)
 
         instances = self.filter_instances(args)
 
@@ -221,23 +253,25 @@ class Runner(ABC):
             "size": instance.size,
         }
 
+        default_props = {
+            "iterations": 0,
+            "num_accepted_steps": 0,
+            "final_scaled_obj": 0.0,
+            "final_stat_res": 0.0,
+            "final_cons_violation": 0.0,
+            "dist_factor": 0.0,
+        }
+
         if result == "timeout":
             return {
                 **info,
                 "status": "timeout",
                 "total_time": args.time_limit,
-                "iterations": 0,
-                "num_accepted_steps": 0,
+                **default_props,
             }
 
         elif result == "error":
-            return {
-                **info,
-                "status": "error",
-                "total_time": 0.0,
-                "iterations": 0,
-                "num_accepted_steps": 0,
-            }
+            return {**info, "status": "error", "total_time": 0.0, **default_props}
         else:
             return {
                 **info,
@@ -245,18 +279,26 @@ class Runner(ABC):
                 "total_time": result.total_time,
                 "iterations": result.iterations,
                 "num_accepted_steps": result.num_accepted_steps,
+                "final_scaled_obj": result.final_scaled_obj,
+                "final_stat_res": result.final_stat_res,
+                "final_cons_violation": result.final_cons_violation,
+                "dist_factor": result.dist_factor,
             }
 
     def solve(self, instances, args):
         import csv
 
+        with_output = not (args.no_output)
+
         params = self.create_params(args)
+        filename = None
 
-        params.write(self.output_filename(args, "params.yml"))
+        if with_output:
+            params.write(self.output_filename(args, "params.yml"))
 
-        filename = self.output_filename(args, "output.csv")
+            filename = self.output_filename(args, "output.csv")
 
-        run_logger.info("Writing results to '%s'", filename)
+            run_logger.info("Writing results to '%s'", filename)
 
         fieldnames = [
             "instance",
@@ -267,13 +309,22 @@ class Runner(ABC):
             "total_time",
             "iterations",
             "num_accepted_steps",
+            "final_scaled_obj",
+            "final_stat_res",
+            "final_cons_violation",
+            "dist_factor",
         ]
 
-        with open(filename, "w") as output_file:
-            writer = csv.DictWriter(output_file, fieldnames=fieldnames)
-            writer.writeheader()
+        if with_output:
+            with open(filename, "w") as output_file:
+                writer = csv.DictWriter(output_file, fieldnames=fieldnames)
+                writer.writeheader()
 
+                for instance, result in self.solve_instances(instances, args):
+                    run_logger.info("Finished instance %s", instance.name)
+                    writer.writerow(self.create_csv_row(args, instance, result))
+                    output_file.flush()
+
+        else:
             for instance, result in self.solve_instances(instances, args):
                 run_logger.info("Finished instance %s", instance.name)
-                writer.writerow(self.create_csv_row(args, instance, result))
-                output_file.flush()
